@@ -437,44 +437,44 @@ typedef ptrdiff_t nghttp3_ssize;
 /**
  * @functypedef
  *
- * Custom memory allocator to replace :manpage:`malloc(3)`.  The
- * |mem_user_data| is the mem_user_data member of :type:`nghttp3_mem`
- * structure.
+ * :type:`nghttp3_malloc` is a custom memory allocator to replace
+ * :manpage:`malloc(3)`.  The |user_data| is the
+ * :member:`nghttp3_mem.user_data`.
  */
-typedef void *(*nghttp3_malloc)(size_t size, void *mem_user_data);
+typedef void *(*nghttp3_malloc)(size_t size, void *user_data);
 
 /**
  * @functypedef
  *
- * Custom memory allocator to replace :manpage:`free(3)`.  The
- * |mem_user_data| is the mem_user_data member of :type:`nghttp3_mem`
- * structure.
+ * :type:`nghttp3_free` is a custom memory allocator to replace
+ * :manpage:`free(3)`.  The |user_data| is the
+ * :member:`nghttp3_mem.user_data`.
  */
-typedef void (*nghttp3_free)(void *ptr, void *mem_user_data);
+typedef void (*nghttp3_free)(void *ptr, void *user_data);
 
 /**
  * @functypedef
  *
- * Custom memory allocator to replace :manpage:`calloc(3)`.  The
- * |mem_user_data| is the mem_user_data member of :type:`nghttp3_mem`
- * structure.
+ * :type:`nghttp3_calloc` is a custom memory allocator to replace
+ * :manpage:`calloc(3)`.  The |user_data| is the
+ * :member:`nghttp3_mem.user_data`.
  */
-typedef void *(*nghttp3_calloc)(size_t nmemb, size_t size, void *mem_user_data);
+typedef void *(*nghttp3_calloc)(size_t nmemb, size_t size, void *user_data);
 
 /**
  * @functypedef
  *
- * Custom memory allocator to replace :manpage:`realloc(3)`.  The
- * |mem_user_data| is the mem_user_data member of :type:`nghttp3_mem`
- * structure.
+ * :type:`nghttp3_realloc` is a custom memory allocator to replace
+ * :manpage:`realloc(3)`.  The |user_data| is the
+ * :member:`nghttp3_mem.user_data`.
  */
-typedef void *(*nghttp3_realloc)(void *ptr, size_t size, void *mem_user_data);
+typedef void *(*nghttp3_realloc)(void *ptr, size_t size, void *user_data);
 
 /**
  * @struct
  *
  * :type:`nghttp3_mem` is a custom memory allocator functions and user
- * defined pointer.  The |mem_user_data| member is passed to each
+ * defined pointer.  The :member:`user_data` field is passed to each
  * allocator function.  This can be used, for example, to achieve
  * per-session memory pool.
  *
@@ -483,17 +483,23 @@ typedef void *(*nghttp3_realloc)(void *ptr, size_t size, void *mem_user_data);
  * standard allocators :manpage:`malloc(3)`, :manpage:`free(3)`,
  * :manpage:`calloc(3)` and :manpage:`realloc(3)` respectively::
  *
- *     void *my_malloc_cb(size_t size, void *mem_user_data) {
+ *     void *my_malloc_cb(size_t size, void *user_data) {
+ *       (void)user_data;
  *       return my_malloc(size);
  *     }
  *
- *     void my_free_cb(void *ptr, void *mem_user_data) { my_free(ptr); }
+ *     void my_free_cb(void *ptr, void *user_data) {
+ *       (void)user_data;
+ *       my_free(ptr);
+ *     }
  *
- *     void *my_calloc_cb(size_t nmemb, size_t size, void *mem_user_data) {
+ *     void *my_calloc_cb(size_t nmemb, size_t size, void *user_data) {
+ *       (void)user_data;
  *       return my_calloc(nmemb, size);
  *     }
  *
- *     void *my_realloc_cb(void *ptr, size_t size, void *mem_user_data) {
+ *     void *my_realloc_cb(void *ptr, size_t size, void *user_data) {
+ *       (void)user_data;
  *       return my_realloc(ptr, size);
  *     }
  *
@@ -506,10 +512,10 @@ typedef void *(*nghttp3_realloc)(void *ptr, size_t size, void *mem_user_data);
  */
 typedef struct nghttp3_mem {
   /**
-   * :member:`mem_user_data` is an arbitrary user supplied data.  This
+   * :member:`user_data` is an arbitrary user supplied data.  This
    * is passed to each allocator function.
    */
-  void *mem_user_data;
+  void *user_data;
   /**
    * :member:`malloc` is a custom allocator function to replace
    * :manpage:`malloc(3)`.
@@ -687,7 +693,7 @@ NGHTTP3_EXTERN void nghttp3_buf_reset(nghttp3_buf *buf);
  *
  * :macro:`NGHTTP3_NV_FLAG_NONE` indicates no flag set.
  */
-#define NGHTTP3_NV_FLAG_NONE 0
+#define NGHTTP3_NV_FLAG_NONE 0x00u
 
 /**
  * @macro
@@ -696,7 +702,7 @@ NGHTTP3_EXTERN void nghttp3_buf_reset(nghttp3_buf *buf);
  * pair must not be indexed.  Other implementation calls this bit as
  * "sensitive".
  */
-#define NGHTTP3_NV_FLAG_NEVER_INDEX 0x01
+#define NGHTTP3_NV_FLAG_NEVER_INDEX 0x01u
 
 /**
  * @macro
@@ -705,7 +711,7 @@ NGHTTP3_EXTERN void nghttp3_buf_reset(nghttp3_buf *buf);
  * If this flag is set, the library does not make a copy of header
  * field name.  This could improve performance.
  */
-#define NGHTTP3_NV_FLAG_NO_COPY_NAME 0x02
+#define NGHTTP3_NV_FLAG_NO_COPY_NAME 0x02u
 
 /**
  * @macro
@@ -714,7 +720,7 @@ NGHTTP3_EXTERN void nghttp3_buf_reset(nghttp3_buf *buf);
  * application.  If this flag is set, the library does not make a copy
  * of header field value.  This could improve performance.
  */
-#define NGHTTP3_NV_FLAG_NO_COPY_VALUE 0x04
+#define NGHTTP3_NV_FLAG_NO_COPY_VALUE 0x04u
 
 /**
  * @struct
@@ -743,7 +749,7 @@ typedef struct nghttp3_nv {
   size_t valuelen;
   /**
    * :member:`flags` is bitwise OR of one or more of
-   * NGHTTP3_NV_FLAG_*.
+   * :macro:`NGHTTP3_NV_FLAG_* <NGHTTP3_NV_FLAG_NONE>`.
    */
   uint8_t flags;
 } nghttp3_nv;
@@ -1071,7 +1077,7 @@ typedef struct nghttp3_qpack_nv {
   int32_t token;
   /**
    * :member:`flags` is a bitwise OR of one or more of
-   * NGHTTP3_NV_FLAG_*.  See :macro:`NGHTTP3_NV_FLAG_NONE`.
+   * :macro:`NGHTTP3_NV_FLAG_* <NGHTTP3_NV_FLAG_NONE>`.
    */
   uint8_t flags;
 } nghttp3_qpack_nv;
@@ -1343,7 +1349,7 @@ nghttp3_qpack_decoder_get_icnt(const nghttp3_qpack_decoder *decoder);
  *
  * :macro:`NGHTTP3_QPACK_DECODE_FLAG_NONE` indicates that no flag set.
  */
-#define NGHTTP3_QPACK_DECODE_FLAG_NONE 0
+#define NGHTTP3_QPACK_DECODE_FLAG_NONE 0x00u
 
 /**
  * @macro
@@ -1351,7 +1357,7 @@ nghttp3_qpack_decoder_get_icnt(const nghttp3_qpack_decoder *decoder);
  * :macro:`NGHTTP3_QPACK_DECODE_FLAG_EMIT` indicates that a header
  * field is successfully decoded.
  */
-#define NGHTTP3_QPACK_DECODE_FLAG_EMIT 0x01
+#define NGHTTP3_QPACK_DECODE_FLAG_EMIT 0x01u
 
 /**
  * @macro
@@ -1359,7 +1365,7 @@ nghttp3_qpack_decoder_get_icnt(const nghttp3_qpack_decoder *decoder);
  * :macro:`NGHTTP3_QPACK_DECODE_FLAG_FINAL` indicates that all header
  * fields have been decoded.
  */
-#define NGHTTP3_QPACK_DECODE_FLAG_FINAL 0x02
+#define NGHTTP3_QPACK_DECODE_FLAG_FINAL 0x02u
 
 /**
  * @macro
@@ -1367,7 +1373,7 @@ nghttp3_qpack_decoder_get_icnt(const nghttp3_qpack_decoder *decoder);
  * :macro:`NGHTTP3_QPACK_DECODE_FLAG_BLOCKED` indicates that decoding
  * has been blocked.
  */
-#define NGHTTP3_QPACK_DECODE_FLAG_BLOCKED 0x04
+#define NGHTTP3_QPACK_DECODE_FLAG_BLOCKED 0x04u
 
 /**
  * @function
@@ -1393,7 +1399,13 @@ nghttp3_qpack_decoder_get_icnt(const nghttp3_qpack_decoder *decoder);
  * Therefore, when application finishes processing the header field,
  * it must call `nghttp3_rcbuf_decref(nv->name)
  * <nghttp3_rcbuf_decref>` and `nghttp3_rcbuf_decref(nv->value)
- * <nghttp3_rcbuf_decref>` or memory leak might occur.
+ * <nghttp3_rcbuf_decref>` or memory leak might occur.  These
+ * :type:`nghttp3_rcbuf` objects hold the pointer to
+ * :type:`nghttp3_mem` that is passed to `nghttp3_qpack_decoder_new`
+ * (or either `nghttp3_conn_client_new` or `nghttp3_conn_server_new`
+ * if it is used indirectly).  As long as these objects are alive, the
+ * pointed :type:`nghttp3_mem` object must be available.  Otherwise,
+ * `nghttp3_rcbuf_decref` will cause undefined behavior.
  *
  * This function returns the number of bytes read, or one of the
  * following negative error codes:
@@ -1674,7 +1686,7 @@ typedef int (*nghttp3_begin_headers)(nghttp3_conn *conn, int64_t stream_id,
  * |name| contains a field name and |value| contains a field value.
  * |token| is one of token defined in :type:`nghttp3_qpack_token` or
  * -1 if no token is defined for |name|.  |flags| is bitwise OR of
- * zero or more of NGHTTP3_NV_FLAG_*.
+ * zero or more of :macro:`NGHTTP3_NV_FLAG_* <NGHTTP3_NV_FLAG_NONE>`.
  *
  * The buffers for |name| and |value| are reference counted. If
  * application needs to keep them, increment the reference count with
@@ -1867,7 +1879,7 @@ typedef struct nghttp3_callbacks {
   /**
    * :member:`reset_stream` is a callback function which is invoked
    * when the library asks application to reset stream (by sending
-   * RESET_STREAM).  It is only used by server to reject a request.
+   * RESET_STREAM).
    */
   nghttp3_reset_stream reset_stream;
   /**
@@ -1913,8 +1925,8 @@ typedef struct nghttp3_settings {
   /**
    * :member:`enable_connect_protocol`, if set to nonzero, enables
    * Extended CONNECT Method (see
-   * https://www.ietf.org/archive/id/draft-ietf-httpbis-h3-websockets-00.html).
-   * Client ignores this field.
+   * https://datatracker.ietf.org/doc/html/rfc9220).  Client ignores
+   * this field.
    */
   int enable_connect_protocol;
 } nghttp3_settings;
@@ -2088,8 +2100,8 @@ NGHTTP3_EXTERN int nghttp3_conn_add_ack_offset(nghttp3_conn *conn,
  * `nghttp3_conn_block_stream` tells the library that stream
  * identified by |stream_id| is blocked due to QUIC flow control.
  */
-NGHTTP3_EXTERN int nghttp3_conn_block_stream(nghttp3_conn *conn,
-                                             int64_t stream_id);
+NGHTTP3_EXTERN void nghttp3_conn_block_stream(nghttp3_conn *conn,
+                                              int64_t stream_id);
 
 /**
  * @function
@@ -2101,6 +2113,26 @@ NGHTTP3_EXTERN int nghttp3_conn_block_stream(nghttp3_conn *conn,
 NGHTTP3_EXTERN int nghttp3_conn_unblock_stream(nghttp3_conn *conn,
                                                int64_t stream_id);
 
+/**
+ * @function
+ *
+ * `nghttp3_conn_is_stream_writable` returns nonzero if a stream
+ * identified by |stream_id| is writable.  It is not writable if:
+ *
+ * - the stream does not exist; or,
+ * - the stream is closed (e.g., `nghttp3_conn_close_stream` is
+ *   called); or,
+ * - the stream is QUIC flow control blocked (e.g.,
+ *   `nghttp3_conn_block_stream` is called); or,
+ * - the stream is input data blocked (e.g.,
+ *   :macro:`NGHTTP3_ERR_WOULDBLOCK` is returned from
+ *   :type:`nghttp3_read_data_callback`); or,
+ * - the stream is half-closed local (e.g.,
+ *   `nghttp3_conn_shutdown_stream_write` is called).
+ */
+NGHTTP3_EXTERN int nghttp3_conn_is_stream_writable(nghttp3_conn *conn,
+                                                   int64_t stream_id);
+
 /**
  * @function
  *
@@ -2109,8 +2141,8 @@ NGHTTP3_EXTERN int nghttp3_conn_unblock_stream(nghttp3_conn *conn,
  * prohibited.  This works like `nghttp3_conn_block_stream`, but it
  * cannot be unblocked by `nghttp3_conn_unblock_stream`.
  */
-NGHTTP3_EXTERN int nghttp3_conn_shutdown_stream_write(nghttp3_conn *conn,
-                                                      int64_t stream_id);
+NGHTTP3_EXTERN void nghttp3_conn_shutdown_stream_write(nghttp3_conn *conn,
+                                                       int64_t stream_id);
 
 /**
  * @function
@@ -2137,6 +2169,16 @@ NGHTTP3_EXTERN int nghttp3_conn_resume_stream(nghttp3_conn *conn,
  *
  * `nghttp3_conn_close_stream` closes stream identified by
  * |stream_id|.  |app_error_code| is the reason of the closure.
+ *
+ * This function returns 0 if it succeeds, or one of the following
+ * negative error codes:
+ *
+ * :macro:`NGHTTP3_ERR_STREAM_NOT_FOUND`
+ *     Stream not found.
+ * :macro:`NGHTTP3_ERR_H3_CLOSED_CRITICAL_STREAM`
+ *     A critical stream is closed.
+ * :macro:`NGHTTP3_ERR_CALLBACK_FAILURE`
+ *     User callback failed
  */
 NGHTTP3_EXTERN int nghttp3_conn_close_stream(nghttp3_conn *conn,
                                              int64_t stream_id,
@@ -2153,7 +2195,7 @@ NGHTTP3_EXTERN int nghttp3_conn_close_stream(nghttp3_conn *conn,
  *
  * :macro:`NGHTTP3_DATA_FLAG_NONE` indicates no flag set.
  */
-#define NGHTTP3_DATA_FLAG_NONE 0x00
+#define NGHTTP3_DATA_FLAG_NONE 0x00u
 
 /**
  * @macro
@@ -2163,7 +2205,7 @@ NGHTTP3_EXTERN int nghttp3_conn_close_stream(nghttp3_conn *conn,
  * that sending side of stream is closed unless
  * :macro:`NGHTTP3_DATA_FLAG_NO_END_STREAM` is given at the same time.
  */
-#define NGHTTP3_DATA_FLAG_EOF 0x01
+#define NGHTTP3_DATA_FLAG_EOF 0x01u
 
 /**
  * @macro
@@ -2175,7 +2217,7 @@ NGHTTP3_EXTERN int nghttp3_conn_close_stream(nghttp3_conn *conn,
  * `nghttp3_conn_submit_trailers()` has been called, regardless of
  * this flag, the submitted trailer fields are sent.
  */
-#define NGHTTP3_DATA_FLAG_NO_END_STREAM 0x02
+#define NGHTTP3_DATA_FLAG_NO_END_STREAM 0x02u
 
 /**
  * @function
@@ -2336,11 +2378,10 @@ NGHTTP3_EXTERN int nghttp3_conn_set_stream_user_data(nghttp3_conn *conn,
  *
  * `nghttp3_conn_get_frame_payload_left` returns the number of bytes
  * left to read current frame payload for a stream denoted by
- * |stream_id|.  If no such stream is found, it returns
- * :macro:`NGHTTP3_ERR_STREAM_NOT_FOUND`.
+ * |stream_id|.  If no such stream is found, it returns 0.
  */
-NGHTTP3_EXTERN int64_t nghttp3_conn_get_frame_payload_left(nghttp3_conn *conn,
-                                                           int64_t stream_id);
+NGHTTP3_EXTERN uint64_t nghttp3_conn_get_frame_payload_left(nghttp3_conn *conn,
+                                                            int64_t stream_id);
 
 /**
  * @macrosection
