@@ -33,6 +33,7 @@
 #include "nghttp3_err.h"
 #include "nghttp3_conv.h"
 #include "nghttp3_http.h"
+#include "nghttp3_unreachable.h"
 
 /* NGHTTP3_QPACK_ENCODER_MAX_DTABLE_CAPACITY is the upper bound of the
    dynamic table capacity that QPACK encoder is willing to use. */
@@ -56,7 +57,7 @@ static int conn_call_begin_headers(nghttp3_conn *conn, nghttp3_stream *stream) {
     return 0;
   }
 
-  rv = conn->callbacks.begin_headers(conn, stream->node.nid.id, conn->user_data,
+  rv = conn->callbacks.begin_headers(conn, stream->node.id, conn->user_data,
                                      stream->user_data);
   if (rv != 0) {
     /* TODO Allow ignore headers */
@@ -74,8 +75,8 @@ static int conn_call_end_headers(nghttp3_conn *conn, nghttp3_stream *stream,
     return 0;
   }
 
-  rv = conn->callbacks.end_headers(conn, stream->node.nid.id, fin,
-                                   conn->user_data, stream->user_data);
+  rv = conn->callbacks.end_headers(conn, stream->node.id, fin, conn->user_data,
+                                   stream->user_data);
   if (rv != 0) {
     /* TODO Allow ignore headers */
     return NGHTTP3_ERR_CALLBACK_FAILURE;
@@ -92,8 +93,8 @@ static int conn_call_begin_trailers(nghttp3_conn *conn,
     return 0;
   }
 
-  rv = conn->callbacks.begin_trailers(conn, stream->node.nid.id,
-                                      conn->user_data, stream->user_data);
+  rv = conn->callbacks.begin_trailers(conn, stream->node.id, conn->user_data,
+                                      stream->user_data);
   if (rv != 0) {
     /* TODO Allow ignore headers */
     return NGHTTP3_ERR_CALLBACK_FAILURE;
@@ -110,8 +111,8 @@ static int conn_call_end_trailers(nghttp3_conn *conn, nghttp3_stream *stream,
     return 0;
   }
 
-  rv = conn->callbacks.end_trailers(conn, stream->node.nid.id, fin,
-                                    conn->user_data, stream->user_data);
+  rv = conn->callbacks.end_trailers(conn, stream->node.id, fin, conn->user_data,
+                                    stream->user_data);
   if (rv != 0) {
     /* TODO Allow ignore headers */
     return NGHTTP3_ERR_CALLBACK_FAILURE;
@@ -127,7 +128,7 @@ static int conn_call_end_stream(nghttp3_conn *conn, nghttp3_stream *stream) {
     return 0;
   }
 
-  rv = conn->callbacks.end_stream(conn, stream->node.nid.id, conn->user_data,
+  rv = conn->callbacks.end_stream(conn, stream->node.id, conn->user_data,
                                   stream->user_data);
   if (rv != 0) {
     return NGHTTP3_ERR_CALLBACK_FAILURE;
@@ -144,7 +145,7 @@ static int conn_call_stop_sending(nghttp3_conn *conn, nghttp3_stream *stream,
     return 0;
   }
 
-  rv = conn->callbacks.stop_sending(conn, stream->node.nid.id, app_error_code,
+  rv = conn->callbacks.stop_sending(conn, stream->node.id, app_error_code,
                                     conn->user_data, stream->user_data);
   if (rv != 0) {
     return NGHTTP3_ERR_CALLBACK_FAILURE;
@@ -161,7 +162,7 @@ static int conn_call_reset_stream(nghttp3_conn *conn, nghttp3_stream *stream,
     return 0;
   }
 
-  rv = conn->callbacks.reset_stream(conn, stream->node.nid.id, app_error_code,
+  rv = conn->callbacks.reset_stream(conn, stream->node.id, app_error_code,
                                     conn->user_data, stream->user_data);
   if (rv != 0) {
     return NGHTTP3_ERR_CALLBACK_FAILURE;
@@ -179,7 +180,7 @@ static int conn_call_deferred_consume(nghttp3_conn *conn,
     return 0;
   }
 
-  rv = conn->callbacks.deferred_consume(conn, stream->node.nid.id, nconsumed,
+  rv = conn->callbacks.deferred_consume(conn, stream->node.id, nconsumed,
                                         conn->user_data, stream->user_data);
   if (rv != 0) {
     return NGHTTP3_ERR_CALLBACK_FAILURE;
@@ -204,7 +205,7 @@ static int cycle_less(const nghttp3_pq_entry *lhsx,
   const nghttp3_tnode *rhs = nghttp3_struct_of(rhsx, nghttp3_tnode, pe);
 
   if (lhs->cycle == rhs->cycle) {
-    return lhs->seq < rhs->seq;
+    return lhs->id < rhs->id;
   }
 
   return rhs->cycle - lhs->cycle <= NGHTTP3_TNODE_MAX_CYCLE_GAP;
@@ -229,10 +230,11 @@ static int conn_new(nghttp3_conn **pconn, int server, int callbacks_version,
     return NGHTTP3_ERR_NOMEM;
   }
 
-  rv = nghttp3_map_init(&conn->streams, mem);
-  if (rv != 0) {
-    goto streams_init_fail;
-  }
+  nghttp3_objalloc_init(&conn->out_chunk_objalloc,
+                        NGHTTP3_STREAM_MIN_CHUNK_SIZE * 16, mem);
+  nghttp3_objalloc_stream_init(&conn->stream_objalloc, 64, mem);
+
+  nghttp3_map_init(&conn->streams, mem);
 
   rv = nghttp3_qpack_decoder_init(&conn->qdec,
                                   settings->qpack_max_dtable_capacity,
@@ -253,10 +255,7 @@ static int conn_new(nghttp3_conn **pconn, int server, int callbacks_version,
     nghttp3_pq_init(&conn->sched[i].spq, cycle_less, mem);
   }
 
-  rv = nghttp3_idtr_init(&conn->remote.bidi.idtr, server, mem);
-  if (rv != 0) {
-    goto remote_bidi_idtr_init_fail;
-  }
+  nghttp3_idtr_init(&conn->remote.bidi.idtr, server, mem);
 
   conn->callbacks = *callbacks;
   conn->local.settings = *settings;
@@ -266,7 +265,6 @@ static int conn_new(nghttp3_conn **pconn, int server, int callbacks_version,
   nghttp3_settings_default(&conn->remote.settings);
   conn->mem = mem;
   conn->user_data = user_data;
-  conn->next_seq = 0;
   conn->server = server;
   conn->rx.goaway_id = NGHTTP3_VARINT_MAX + 1;
   conn->tx.goaway_id = NGHTTP3_VARINT_MAX + 1;
@@ -276,13 +274,12 @@ static int conn_new(nghttp3_conn **pconn, int server, int callbacks_version,
 
   return 0;
 
-remote_bidi_idtr_init_fail:
-  nghttp3_qpack_encoder_free(&conn->qenc);
 qenc_init_fail:
   nghttp3_qpack_decoder_free(&conn->qdec);
 qdec_init_fail:
   nghttp3_map_free(&conn->streams);
-streams_init_fail:
+  nghttp3_objalloc_free(&conn->stream_objalloc);
+  nghttp3_objalloc_free(&conn->out_chunk_objalloc);
   nghttp3_mem_free(mem, conn);
 
   return rv;
@@ -356,6 +353,9 @@ void nghttp3_conn_del(nghttp3_conn *conn) {
   nghttp3_map_each_free(&conn->streams, free_stream, NULL);
   nghttp3_map_free(&conn->streams);
 
+  nghttp3_objalloc_free(&conn->stream_objalloc);
+  nghttp3_objalloc_free(&conn->out_chunk_objalloc);
+
   nghttp3_mem_free(conn->mem, conn);
 }
 
@@ -454,12 +454,6 @@ nghttp3_ssize nghttp3_conn_read_stream(nghttp3_conn *conn, int64_t stream_id,
         stream->tx.hstate = NGHTTP3_HTTP_STATE_REQ_INITIAL;
       }
     }
-  } else if (nghttp3_stream_uni(stream_id) &&
-             stream->type == NGHTTP3_STREAM_TYPE_PUSH) {
-    if (stream->rx.hstate == NGHTTP3_HTTP_STATE_NONE) {
-      stream->rx.hstate = NGHTTP3_HTTP_STATE_RESP_INITIAL;
-      stream->tx.hstate = NGHTTP3_HTTP_STATE_RESP_INITIAL;
-    }
   }
 
   if (srclen == 0 && !fin) {
@@ -603,8 +597,7 @@ nghttp3_ssize nghttp3_conn_read_uni(nghttp3_conn *conn, nghttp3_stream *stream,
     }
     break;
   default:
-    /* unreachable */
-    assert(0);
+    nghttp3_unreachable();
   }
 
   if (nconsumed < 0) {
@@ -1013,8 +1006,7 @@ nghttp3_ssize nghttp3_conn_read_control(nghttp3_conn *conn,
       nghttp3_stream_read_state_reset(rstate);
       break;
     default:
-      /* unreachable */
-      assert(0);
+      nghttp3_unreachable();
     }
   }
 
@@ -1022,7 +1014,7 @@ nghttp3_ssize nghttp3_conn_read_control(nghttp3_conn *conn,
 }
 
 static int conn_delete_stream(nghttp3_conn *conn, nghttp3_stream *stream) {
-  int bidi = nghttp3_client_stream_bidi(stream->node.nid.id);
+  int bidi = nghttp3_client_stream_bidi(stream->node.id);
   int rv;
 
   rv = conn_call_deferred_consume(conn, stream,
@@ -1032,16 +1024,15 @@ static int conn_delete_stream(nghttp3_conn *conn, nghttp3_stream *stream) {
   }
 
   if (bidi && conn->callbacks.stream_close) {
-    rv = conn->callbacks.stream_close(conn, stream->node.nid.id,
-                                      stream->error_code, conn->user_data,
-                                      stream->user_data);
+    rv = conn->callbacks.stream_close(conn, stream->node.id, stream->error_code,
+                                      conn->user_data, stream->user_data);
     if (rv != 0) {
       return NGHTTP3_ERR_CALLBACK_FAILURE;
     }
   }
 
-  rv = nghttp3_map_remove(&conn->streams,
-                          (nghttp3_map_key_type)stream->node.nid.id);
+  rv =
+      nghttp3_map_remove(&conn->streams, (nghttp3_map_key_type)stream->node.id);
 
   assert(0 == rv);
 
@@ -1058,7 +1049,7 @@ static int conn_process_blocked_stream_data(nghttp3_conn *conn,
   int rv;
   size_t len;
 
-  assert(nghttp3_client_stream_bidi(stream->node.nid.id));
+  assert(nghttp3_client_stream_bidi(stream->node.id));
 
   for (;;) {
     len = nghttp3_ringbuf_len(&stream->inq);
@@ -1150,7 +1141,7 @@ static nghttp3_tnode *stream_get_sched_node(nghttp3_stream *stream) {
 
 static int conn_update_stream_priority(nghttp3_conn *conn,
                                        nghttp3_stream *stream, uint8_t pri) {
-  assert(nghttp3_client_stream_bidi(stream->node.nid.id));
+  assert(nghttp3_client_stream_bidi(stream->node.id));
 
   if (stream->node.pri == pri) {
     return 0;
@@ -1286,8 +1277,7 @@ nghttp3_ssize nghttp3_conn_read_bidi(nghttp3_conn *conn, size_t *pnproc,
           rv = conn_call_begin_trailers(conn, stream);
           break;
         default:
-          /* Unreachable */
-          assert(0);
+          nghttp3_unreachable();
         }
 
         if (rv != 0) {
@@ -1339,6 +1329,10 @@ nghttp3_ssize nghttp3_conn_read_bidi(nghttp3_conn *conn, size_t *pnproc,
       nread = nghttp3_conn_on_headers(conn, stream, p, len,
                                       (int64_t)len == rstate->left);
       if (nread < 0) {
+        if (nread == NGHTTP3_ERR_MALFORMED_HTTP_HEADER) {
+          goto http_header_error;
+        }
+
         return nread;
       }
 
@@ -1373,12 +1367,14 @@ nghttp3_ssize nghttp3_conn_read_bidi(nghttp3_conn *conn, size_t *pnproc,
         rv = 0;
         break;
       default:
-        /* Unreachable */
-        assert(0);
-        abort();
+        nghttp3_unreachable();
       }
 
       if (rv != 0) {
+        if (rv == NGHTTP3_ERR_MALFORMED_HTTP_HEADER) {
+          goto http_header_error;
+        }
+
         return rv;
       }
 
@@ -1387,6 +1383,7 @@ nghttp3_ssize nghttp3_conn_read_bidi(nghttp3_conn *conn, size_t *pnproc,
         /* Only server utilizes priority information to schedule
            streams. */
         if (conn->server &&
+            (stream->rx.http.flags & NGHTTP3_HTTP_FLAG_PRIORITY) &&
             !(stream->flags & NGHTTP3_STREAM_FLAG_PRIORITY_UPDATE_RECVED) &&
             !(stream->flags & NGHTTP3_STREAM_FLAG_SERVER_PRIORITY_SET)) {
           rv = conn_update_stream_priority(conn, stream, stream->rx.http.pri);
@@ -1403,8 +1400,7 @@ nghttp3_ssize nghttp3_conn_read_bidi(nghttp3_conn *conn, size_t *pnproc,
         rv = conn_call_end_trailers(conn, stream, p == end && fin);
         break;
       default:
-        /* Unreachable */
-        assert(0);
+        nghttp3_unreachable();
       }
 
       if (rv != 0) {
@@ -1416,6 +1412,25 @@ nghttp3_ssize nghttp3_conn_read_bidi(nghttp3_conn *conn, size_t *pnproc,
       assert(0 == rv);
 
       nghttp3_stream_read_state_reset(rstate);
+
+      break;
+
+    http_header_error:
+      stream->flags |= NGHTTP3_STREAM_FLAG_HTTP_ERROR;
+
+      busy = 1;
+      rstate->state = NGHTTP3_REQ_STREAM_STATE_IGN_REST;
+
+      rv = conn_call_stop_sending(conn, stream, NGHTTP3_H3_MESSAGE_ERROR);
+      if (rv != 0) {
+        return rv;
+      }
+
+      rv = conn_call_reset_stream(conn, stream, NGHTTP3_H3_MESSAGE_ERROR);
+      if (rv != 0) {
+        return rv;
+      }
+
       break;
     case NGHTTP3_REQ_STREAM_STATE_IGN_FRAME:
       len = (size_t)nghttp3_min(rstate->left, (int64_t)(end - p));
@@ -1431,7 +1446,7 @@ nghttp3_ssize nghttp3_conn_read_bidi(nghttp3_conn *conn, size_t *pnproc,
       break;
     case NGHTTP3_REQ_STREAM_STATE_IGN_REST:
       nconsumed += (size_t)(end - p);
-      *pnproc = (size_t)(p - src);
+      *pnproc = (size_t)(end - src);
       return (nghttp3_ssize)nconsumed;
     }
   }
@@ -1477,7 +1492,7 @@ int nghttp3_conn_on_data(nghttp3_conn *conn, nghttp3_stream *stream,
     return 0;
   }
 
-  rv = conn->callbacks.recv_data(conn, stream->node.nid.id, data, datalen,
+  rv = conn->callbacks.recv_data(conn, stream->node.id, data, datalen,
                                  conn->user_data, stream->user_data);
   if (rv != 0) {
     return NGHTTP3_ERR_CALLBACK_FAILURE;
@@ -1524,8 +1539,7 @@ static nghttp3_ssize conn_decode_headers(nghttp3_conn *conn,
     recv_header = conn->callbacks.recv_trailer;
     break;
   default:
-    /* Unreachable */
-    assert(0);
+    nghttp3_unreachable();
   }
   http = &stream->rx.http;
 
@@ -1578,17 +1592,15 @@ static nghttp3_ssize conn_decode_headers(nghttp3_conn *conn,
         break;
       case 0:
         if (recv_header) {
-          rv = recv_header(conn, stream->node.nid.id, nv.token, nv.name,
-                           nv.value, nv.flags, conn->user_data,
-                           stream->user_data);
+          rv = recv_header(conn, stream->node.id, nv.token, nv.name, nv.value,
+                           nv.flags, conn->user_data, stream->user_data);
           if (rv != 0) {
             rv = NGHTTP3_ERR_CALLBACK_FAILURE;
           }
         }
         break;
       default:
-        /* Unreachable */
-        assert(0);
+        nghttp3_unreachable();
       }
 
       nghttp3_rcbuf_decref(nv.name);
@@ -1649,19 +1661,27 @@ int nghttp3_conn_on_settings_entry_received(nghttp3_conn *conn,
 
     dest->qpack_blocked_streams = (size_t)ent->value;
 
-    nghttp3_qpack_encoder_set_max_blocked_streams(&conn->qenc,
-                                                  nghttp3_min(100, ent->value));
+    nghttp3_qpack_encoder_set_max_blocked_streams(
+        &conn->qenc, (size_t)nghttp3_min(100, ent->value));
     break;
   case NGHTTP3_SETTINGS_ID_ENABLE_CONNECT_PROTOCOL:
     if (!conn->server) {
       break;
     }
-    if (ent->value != 0 && ent->value != 1) {
-      return NGHTTP3_ERR_H3_SETTINGS_ERROR;
-    }
-    if (ent->value == 0 && dest->enable_connect_protocol) {
+
+    switch (ent->value) {
+    case 0:
+      if (dest->enable_connect_protocol) {
+        return NGHTTP3_ERR_H3_SETTINGS_ERROR;
+      }
+
+      break;
+    case 1:
+      break;
+    default:
       return NGHTTP3_ERR_H3_SETTINGS_ERROR;
     }
+
     dest->enable_connect_protocol = (int)ent->value;
     break;
   case NGHTTP3_H2_SETTINGS_ID_ENABLE_PUSH:
@@ -1766,7 +1786,8 @@ int nghttp3_conn_create_stream(nghttp3_conn *conn, nghttp3_stream **pstream,
       conn_stream_acked_data,
   };
 
-  rv = nghttp3_stream_new(&stream, stream_id, conn->next_seq, &callbacks,
+  rv = nghttp3_stream_new(&stream, stream_id, &callbacks,
+                          &conn->out_chunk_objalloc, &conn->stream_objalloc,
                           conn->mem);
   if (rv != 0) {
     return rv;
@@ -1774,14 +1795,13 @@ int nghttp3_conn_create_stream(nghttp3_conn *conn, nghttp3_stream **pstream,
 
   stream->conn = conn;
 
-  rv = nghttp3_map_insert(&conn->streams,
-                          (nghttp3_map_key_type)stream->node.nid.id, stream);
+  rv = nghttp3_map_insert(&conn->streams, (nghttp3_map_key_type)stream->node.id,
+                          stream);
   if (rv != 0) {
     nghttp3_stream_del(stream);
     return rv;
   }
 
-  ++conn->next_seq;
   *pstream = stream;
 
   return 0;
@@ -1881,14 +1901,14 @@ static nghttp3_ssize conn_writev_stream(nghttp3_conn *conn, int64_t *pstream_id,
     }
   }
 
-  if (!nghttp3_stream_uni(stream->node.nid.id) && conn->tx.qenc &&
+  if (!nghttp3_stream_uni(stream->node.id) && conn->tx.qenc &&
       !nghttp3_stream_is_blocked(conn->tx.qenc)) {
     n = nghttp3_stream_writev(conn->tx.qenc, pfin, vec, veccnt);
     if (n < 0) {
       return n;
     }
     if (n) {
-      *pstream_id = conn->tx.qenc->node.nid.id;
+      *pstream_id = conn->tx.qenc->node.id;
       return n;
     }
   }
@@ -1903,7 +1923,7 @@ static nghttp3_ssize conn_writev_stream(nghttp3_conn *conn, int64_t *pstream_id,
     return 0;
   }
 
-  *pstream_id = stream->node.nid.id;
+  *pstream_id = stream->node.id;
 
   return n;
 }
@@ -1961,7 +1981,7 @@ nghttp3_ssize nghttp3_conn_writev_stream(nghttp3_conn *conn,
     return ncnt;
   }
 
-  if (nghttp3_client_stream_bidi(stream->node.nid.id) &&
+  if (nghttp3_client_stream_bidi(stream->node.id) &&
       !nghttp3_stream_require_schedule(stream)) {
     nghttp3_conn_unschedule_stream(conn, stream);
   }
@@ -1994,7 +2014,7 @@ int nghttp3_conn_add_write_offset(nghttp3_conn *conn, int64_t stream_id,
   int rv;
 
   if (stream == NULL) {
-    return NGHTTP3_ERR_STREAM_NOT_FOUND;
+    return 0;
   }
 
   rv = nghttp3_stream_add_outq_offset(stream, n);
@@ -2004,7 +2024,7 @@ int nghttp3_conn_add_write_offset(nghttp3_conn *conn, int64_t stream_id,
 
   stream->unscheduled_nwrite += n;
 
-  if (!nghttp3_client_stream_bidi(stream->node.nid.id)) {
+  if (!nghttp3_client_stream_bidi(stream->node.id)) {
     return 0;
   }
 
@@ -2025,7 +2045,7 @@ int nghttp3_conn_add_ack_offset(nghttp3_conn *conn, int64_t stream_id,
   nghttp3_stream *stream = nghttp3_conn_find_stream(conn, stream_id);
 
   if (stream == NULL) {
-    return NGHTTP3_ERR_STREAM_NOT_FOUND;
+    return 0;
   }
 
   return nghttp3_stream_add_ack_offset(stream, n);
@@ -2266,50 +2286,46 @@ int nghttp3_conn_reject_stream(nghttp3_conn *conn, nghttp3_stream *stream) {
   return conn_call_reset_stream(conn, stream, NGHTTP3_H3_REQUEST_REJECTED);
 }
 
-int nghttp3_conn_block_stream(nghttp3_conn *conn, int64_t stream_id) {
+void nghttp3_conn_block_stream(nghttp3_conn *conn, int64_t stream_id) {
   nghttp3_stream *stream = nghttp3_conn_find_stream(conn, stream_id);
 
   if (stream == NULL) {
-    return NGHTTP3_ERR_STREAM_NOT_FOUND;
+    return;
   }
 
   stream->flags |= NGHTTP3_STREAM_FLAG_FC_BLOCKED;
   stream->unscheduled_nwrite = 0;
 
-  if (nghttp3_client_stream_bidi(stream->node.nid.id)) {
+  if (nghttp3_client_stream_bidi(stream->node.id)) {
     nghttp3_conn_unschedule_stream(conn, stream);
   }
-
-  return 0;
 }
 
-int nghttp3_conn_shutdown_stream_write(nghttp3_conn *conn, int64_t stream_id) {
+void nghttp3_conn_shutdown_stream_write(nghttp3_conn *conn, int64_t stream_id) {
   nghttp3_stream *stream = nghttp3_conn_find_stream(conn, stream_id);
 
   if (stream == NULL) {
-    return NGHTTP3_ERR_STREAM_NOT_FOUND;
+    return;
   }
 
   stream->flags |= NGHTTP3_STREAM_FLAG_SHUT_WR;
   stream->unscheduled_nwrite = 0;
 
-  if (nghttp3_client_stream_bidi(stream->node.nid.id)) {
+  if (nghttp3_client_stream_bidi(stream->node.id)) {
     nghttp3_conn_unschedule_stream(conn, stream);
   }
-
-  return 0;
 }
 
 int nghttp3_conn_unblock_stream(nghttp3_conn *conn, int64_t stream_id) {
   nghttp3_stream *stream = nghttp3_conn_find_stream(conn, stream_id);
 
   if (stream == NULL) {
-    return NGHTTP3_ERR_STREAM_NOT_FOUND;
+    return 0;
   }
 
   stream->flags &= (uint16_t)~NGHTTP3_STREAM_FLAG_FC_BLOCKED;
 
-  if (nghttp3_client_stream_bidi(stream->node.nid.id) &&
+  if (nghttp3_client_stream_bidi(stream->node.id) &&
       nghttp3_stream_require_schedule(stream)) {
     return nghttp3_conn_ensure_stream_scheduled(conn, stream);
   }
@@ -2317,16 +2333,29 @@ int nghttp3_conn_unblock_stream(nghttp3_conn *conn, int64_t stream_id) {
   return 0;
 }
 
+int nghttp3_conn_is_stream_writable(nghttp3_conn *conn, int64_t stream_id) {
+  nghttp3_stream *stream = nghttp3_conn_find_stream(conn, stream_id);
+
+  if (stream == NULL) {
+    return 0;
+  }
+
+  return (stream->flags &
+          (NGHTTP3_STREAM_FLAG_FC_BLOCKED |
+           NGHTTP3_STREAM_FLAG_READ_DATA_BLOCKED | NGHTTP3_STREAM_FLAG_SHUT_WR |
+           NGHTTP3_STREAM_FLAG_CLOSED)) == 0;
+}
+
 int nghttp3_conn_resume_stream(nghttp3_conn *conn, int64_t stream_id) {
   nghttp3_stream *stream = nghttp3_conn_find_stream(conn, stream_id);
 
   if (stream == NULL) {
-    return NGHTTP3_ERR_STREAM_NOT_FOUND;
+    return 0;
   }
 
   stream->flags &= (uint16_t)~NGHTTP3_STREAM_FLAG_READ_DATA_BLOCKED;
 
-  if (nghttp3_client_stream_bidi(stream->node.nid.id) &&
+  if (nghttp3_client_stream_bidi(stream->node.id) &&
       nghttp3_stream_require_schedule(stream)) {
     return nghttp3_conn_ensure_stream_scheduled(conn, stream);
   }
@@ -2343,7 +2372,6 @@ int nghttp3_conn_close_stream(nghttp3_conn *conn, int64_t stream_id,
   }
 
   if (nghttp3_stream_uni(stream_id) &&
-      stream->type != NGHTTP3_STREAM_TYPE_PUSH &&
       stream->type != NGHTTP3_STREAM_TYPE_UNKNOWN) {
     return NGHTTP3_ERR_H3_CLOSED_CRITICAL_STREAM;
   }
@@ -2419,15 +2447,15 @@ int nghttp3_conn_set_stream_user_data(nghttp3_conn *conn, int64_t stream_id,
   return 0;
 }
 
-int64_t nghttp3_conn_get_frame_payload_left(nghttp3_conn *conn,
-                                            int64_t stream_id) {
+uint64_t nghttp3_conn_get_frame_payload_left(nghttp3_conn *conn,
+                                             int64_t stream_id) {
   nghttp3_stream *stream = nghttp3_conn_find_stream(conn, stream_id);
 
   if (stream == NULL) {
-    return NGHTTP3_ERR_STREAM_NOT_FOUND;
+    return 0;
   }
 
-  return stream->rstate.left;
+  return (uint64_t)stream->rstate.left;
 }
 
 int nghttp3_conn_get_stream_priority(nghttp3_conn *conn, nghttp3_pri *dest,
