@@ -31,17 +31,23 @@
 #include "nghttp3_stream.h"
 #include "nghttp3_macro.h"
 #include "nghttp3_conv.h"
+#include "nghttp3_unreachable.h"
 
 static uint8_t downcase(uint8_t c) {
   return 'A' <= c && c <= 'Z' ? (uint8_t)(c - 'A' + 'a') : c;
 }
 
+/*
+ * memieq returns 1 if the data pointed by |a| of length |n| equals to
+ * |b| of the same length in case-insensitive manner.  The data
+ * pointed by |a| must not include upper cased letters (A-Z).
+ */
 static int memieq(const void *a, const void *b, size_t n) {
   size_t i;
   const uint8_t *aa = a, *bb = b;
 
   for (i = 0; i < n; ++i) {
-    if (downcase(aa[i]) != downcase(bb[i])) {
+    if (aa[i] != downcase(bb[i])) {
       return 0;
     }
   }
@@ -73,25 +79,12 @@ static int64_t parse_uint(const uint8_t *s, size_t len) {
   return n;
 }
 
-static int lws(const uint8_t *s, size_t n) {
-  size_t i;
-  for (i = 0; i < n; ++i) {
-    if (s[i] != ' ' && s[i] != '\t') {
-      return 0;
-    }
-  }
-  return 1;
-}
-
 static int check_pseudo_header(nghttp3_http_state *http,
-                               const nghttp3_qpack_nv *nv, int flag) {
-  if (http->flags & flag) {
+                               const nghttp3_qpack_nv *nv, uint32_t flag) {
+  if ((http->flags & flag) || nv->value->len == 0) {
     return 0;
   }
-  if (lws(nv->value->base, nv->value->len)) {
-    return 0;
-  }
-  http->flags = (uint16_t)(http->flags | flag);
+  http->flags |= flag;
   return 1;
 }
 
@@ -267,8 +260,7 @@ fin:
 
     return p - begin;
   default:
-    assert(0);
-    abort();
+    nghttp3_unreachable();
   }
 }
 
@@ -887,8 +879,10 @@ static int http_request_on_header(nghttp3_http_state *http,
     if (!check_pseudo_header(http, nv, NGHTTP3_HTTP_FLAG__SCHEME)) {
       return NGHTTP3_ERR_MALFORMED_HTTP_HEADER;
     }
-    if ((nv->value->len == 4 && memieq("http", nv->value->base, 4)) ||
-        (nv->value->len == 5 && memieq("https", nv->value->base, 5))) {
+    /* scheme is case-insensitive:
+       https://datatracker.ietf.org/doc/html/rfc3986#section-3.1 */
+    if (lstrieq("http", nv->value->base, nv->value->len) ||
+        lstrieq("https", nv->value->base, nv->value->len)) {
       http->flags |= NGHTTP3_HTTP_FLAG_SCHEME_HTTP;
     }
     break;
@@ -936,11 +930,17 @@ static int http_request_on_header(nghttp3_http_state *http,
     }
     break;
   case NGHTTP3_QPACK_TOKEN_PRIORITY:
-    pri.urgency = nghttp3_pri_uint8_urgency(http->pri);
-    pri.inc = nghttp3_pri_uint8_inc(http->pri);
-    if (nghttp3_http_parse_priority(&pri, nv->value->base, nv->value->len) ==
-        0) {
-      http->pri = nghttp3_pri_to_uint8(&pri);
+    if (!trailers && !(http->flags & NGHTTP3_HTTP_FLAG_BAD_PRIORITY)) {
+      pri.urgency = nghttp3_pri_uint8_urgency(http->pri);
+      pri.inc = nghttp3_pri_uint8_inc(http->pri);
+      if (nghttp3_http_parse_priority(&pri, nv->value->base, nv->value->len) ==
+          0) {
+        http->pri = nghttp3_pri_to_uint8(&pri);
+        http->flags |= NGHTTP3_HTTP_FLAG_PRIORITY;
+      } else {
+        http->flags &= ~NGHTTP3_HTTP_FLAG_PRIORITY;
+        http->flags |= NGHTTP3_HTTP_FLAG_BAD_PRIORITY;
+      }
     }
     break;
   default:
@@ -1331,7 +1331,13 @@ int nghttp3_http_on_header(nghttp3_http_state *http, nghttp3_qpack_nv *nv,
     break;
   case NGHTTP3_QPACK_TOKEN__AUTHORITY:
   case NGHTTP3_QPACK_TOKEN_HOST:
-    rv = check_authority(nv->value->base, nv->value->len);
+    if (request) {
+      rv = check_authority(nv->value->base, nv->value->len);
+    } else {
+      /* The use of host field in response field section is
+         undefined. */
+      rv = nghttp3_check_header_value(nv->value->base, nv->value->len);
+    }
     break;
   case NGHTTP3_QPACK_TOKEN__PATH:
     rv = check_path(nv->value->base, nv->value->len);
@@ -1404,15 +1410,14 @@ int nghttp3_http_on_response_headers(nghttp3_http_state *http) {
 
   if (http->status_code / 100 == 1) {
     /* non-final response */
-    http->flags = (uint16_t)((http->flags & NGHTTP3_HTTP_FLAG_METH_ALL) |
-                             NGHTTP3_HTTP_FLAG_EXPECT_FINAL_RESPONSE);
+    http->flags = (http->flags & NGHTTP3_HTTP_FLAG_METH_ALL) |
+                  NGHTTP3_HTTP_FLAG_EXPECT_FINAL_RESPONSE;
     http->content_length = -1;
     http->status_code = -1;
     return 0;
   }
 
-  http->flags =
-      (uint16_t)(http->flags & ~NGHTTP3_HTTP_FLAG_EXPECT_FINAL_RESPONSE);
+  http->flags &= ~NGHTTP3_HTTP_FLAG_EXPECT_FINAL_RESPONSE;
 
   if (!expect_response_body(http)) {
     http->content_length = 0;
@@ -1627,6 +1632,19 @@ static const int VALID_HD_VALUE_CHARS[] = {
 
 int nghttp3_check_header_value(const uint8_t *value, size_t len) {
   const uint8_t *last;
+
+  switch (len) {
+  case 0:
+    return 1;
+  case 1:
+    return !(*value == ' ' || *value == '\t');
+  default:
+    if (*value == ' ' || *value == '\t' || *(value + len - 1) == ' ' ||
+        *(value + len - 1) == '\t') {
+      return 0;
+    }
+  }
+
   for (last = value + len; value != last; ++value) {
     if (!VALID_HD_VALUE_CHARS[*value]) {
       return 0;
