@@ -35,6 +35,7 @@
 #include "nghttp3_str.h"
 #include "nghttp3_http.h"
 #include "nghttp3_vec.h"
+#include "nghttp3_unreachable.h"
 
 /* NGHTTP3_STREAM_MAX_COPY_THRES is the maximum size of buffer which
    makes a copy to outq. */
@@ -44,40 +45,27 @@
 #define NGHTTP3_MIN_RBLEN 4
 
 int nghttp3_stream_new(nghttp3_stream **pstream, int64_t stream_id,
-                       uint64_t seq, const nghttp3_stream_callbacks *callbacks,
+                       const nghttp3_stream_callbacks *callbacks,
+                       nghttp3_objalloc *out_chunk_objalloc,
+                       nghttp3_objalloc *stream_objalloc,
                        const nghttp3_mem *mem) {
-  int rv;
-  nghttp3_stream *stream = nghttp3_mem_calloc(mem, 1, sizeof(nghttp3_stream));
-  nghttp3_node_id nid;
+  nghttp3_stream *stream = nghttp3_objalloc_stream_get(stream_objalloc);
 
   if (stream == NULL) {
     return NGHTTP3_ERR_NOMEM;
   }
 
-  nghttp3_tnode_init(
-      &stream->node,
-      nghttp3_node_id_init(&nid, NGHTTP3_NODE_ID_TYPE_STREAM, stream_id), seq,
-      NGHTTP3_DEFAULT_URGENCY);
+  memset(stream, 0, sizeof(*stream));
 
-  rv = nghttp3_ringbuf_init(&stream->frq, 0, sizeof(nghttp3_frame_entry), mem);
-  if (rv != 0) {
-    goto frq_init_fail;
-  }
+  stream->out_chunk_objalloc = out_chunk_objalloc;
+  stream->stream_objalloc = stream_objalloc;
 
-  rv = nghttp3_ringbuf_init(&stream->chunks, 0, sizeof(nghttp3_buf), mem);
-  if (rv != 0) {
-    goto chunks_init_fail;
-  }
+  nghttp3_tnode_init(&stream->node, stream_id, NGHTTP3_DEFAULT_URGENCY);
 
-  rv = nghttp3_ringbuf_init(&stream->outq, 0, sizeof(nghttp3_typed_buf), mem);
-  if (rv != 0) {
-    goto outq_init_fail;
-  }
-
-  rv = nghttp3_ringbuf_init(&stream->inq, 0, sizeof(nghttp3_buf), mem);
-  if (rv != 0) {
-    goto inq_init_fail;
-  }
+  nghttp3_ringbuf_init(&stream->frq, 0, sizeof(nghttp3_frame_entry), mem);
+  nghttp3_ringbuf_init(&stream->chunks, 0, sizeof(nghttp3_buf), mem);
+  nghttp3_ringbuf_init(&stream->outq, 0, sizeof(nghttp3_typed_buf), mem);
+  nghttp3_ringbuf_init(&stream->inq, 0, sizeof(nghttp3_buf), mem);
 
   nghttp3_qpack_stream_context_init(&stream->qpack_sctx, stream_id, mem);
 
@@ -96,17 +84,6 @@ int nghttp3_stream_new(nghttp3_stream **pstream, int64_t stream_id,
   *pstream = stream;
 
   return 0;
-
-inq_init_fail:
-  nghttp3_ringbuf_free(&stream->outq);
-outq_init_fail:
-  nghttp3_ringbuf_free(&stream->chunks);
-chunks_init_fail:
-  nghttp3_ringbuf_free(&stream->frq);
-frq_init_fail:
-  nghttp3_mem_free(mem, stream);
-
-  return rv;
 }
 
 static void delete_outq(nghttp3_ringbuf *outq, const nghttp3_mem *mem) {
@@ -135,6 +112,27 @@ static void delete_chunks(nghttp3_ringbuf *chunks, const nghttp3_mem *mem) {
   nghttp3_ringbuf_free(chunks);
 }
 
+static void delete_out_chunks(nghttp3_ringbuf *chunks,
+                              nghttp3_objalloc *out_chunk_objalloc,
+                              const nghttp3_mem *mem) {
+  nghttp3_buf *buf;
+  size_t i, len = nghttp3_ringbuf_len(chunks);
+
+  for (i = 0; i < len; ++i) {
+    buf = nghttp3_ringbuf_get(chunks, i);
+
+    if (nghttp3_buf_cap(buf) == NGHTTP3_STREAM_MIN_CHUNK_SIZE) {
+      nghttp3_objalloc_chunk_release(out_chunk_objalloc,
+                                     (nghttp3_chunk *)(void *)buf->begin);
+      continue;
+    }
+
+    nghttp3_buf_free(buf, mem);
+  }
+
+  nghttp3_ringbuf_free(chunks);
+}
+
 static void delete_frq(nghttp3_ringbuf *frq, const nghttp3_mem *mem) {
   nghttp3_frame_entry *frent;
   size_t i, len = nghttp3_ringbuf_len(frq);
@@ -161,11 +159,11 @@ void nghttp3_stream_del(nghttp3_stream *stream) {
   nghttp3_qpack_stream_context_free(&stream->qpack_sctx);
   delete_chunks(&stream->inq, stream->mem);
   delete_outq(&stream->outq, stream->mem);
-  delete_chunks(&stream->chunks, stream->mem);
+  delete_out_chunks(&stream->chunks, stream->out_chunk_objalloc, stream->mem);
   delete_frq(&stream->frq, stream->mem);
   nghttp3_tnode_free(&stream->node);
 
-  nghttp3_mem_free(stream->mem, stream);
+  nghttp3_objalloc_stream_release(stream->stream_objalloc, stream);
 }
 
 void nghttp3_varint_read_state_reset(nghttp3_varint_read_state *rvint) {
@@ -187,7 +185,7 @@ nghttp3_ssize nghttp3_read_varint(nghttp3_varint_read_state *rvint,
   if (rvint->left == 0) {
     assert(rvint->acc == 0);
 
-    rvint->left = nghttp3_get_varint_len(src);
+    rvint->left = nghttp3_get_varintlen(src);
     if (rvint->left <= srclen) {
       rvint->acc = nghttp3_get_varint(&nread, src);
       rvint->left = 0;
@@ -307,7 +305,7 @@ static void typed_buf_shared_init(nghttp3_typed_buf *tbuf,
 }
 
 int nghttp3_stream_write_stream_type(nghttp3_stream *stream) {
-  size_t len = nghttp3_put_varint_len((int64_t)stream->type);
+  size_t len = nghttp3_put_varintlen((int64_t)stream->type);
   nghttp3_buf *chunk;
   nghttp3_typed_buf tbuf;
   int rv;
@@ -452,8 +450,8 @@ int nghttp3_stream_write_header_block(nghttp3_stream *stream,
 
   nghttp3_buf_wrap_init(&pbuf, raw_pbuf, sizeof(raw_pbuf));
 
-  rv = nghttp3_qpack_encoder_encode(qenc, &pbuf, rbuf, ebuf,
-                                    stream->node.nid.id, nva, nvlen);
+  rv = nghttp3_qpack_encoder_encode(qenc, &pbuf, rbuf, ebuf, stream->node.id,
+                                    nva, nvlen);
   if (rv != 0) {
     goto fail;
   }
@@ -573,8 +571,8 @@ int nghttp3_stream_write_data(nghttp3_stream *stream, int *peof,
 
   *peof = 0;
 
-  sveccnt = read_data(conn, stream->node.nid.id, vec, nghttp3_arraylen(vec),
-                      &flags, conn->user_data, stream->user_data);
+  sveccnt = read_data(conn, stream->node.id, vec, nghttp3_arraylen(vec), &flags,
+                      conn->user_data, stream->user_data);
   if (sveccnt < 0) {
     if (sveccnt == NGHTTP3_ERR_WOULDBLOCK) {
       stream->flags |= NGHTTP3_STREAM_FLAG_READ_DATA_BLOCKED;
@@ -761,7 +759,12 @@ int nghttp3_stream_ensure_chunk(nghttp3_stream *stream, size_t need) {
   for (; n < need; n *= 2)
     ;
 
-  p = nghttp3_mem_malloc(stream->mem, n);
+  if (n == NGHTTP3_STREAM_MIN_CHUNK_SIZE) {
+    p = (uint8_t *)nghttp3_objalloc_chunk_len_get(stream->out_chunk_objalloc,
+                                                  n);
+  } else {
+    p = nghttp3_mem_malloc(stream->mem, n);
+  }
   if (p == NULL) {
     return NGHTTP3_ERR_NOMEM;
   }
@@ -807,7 +810,7 @@ nghttp3_ssize nghttp3_stream_writev(nghttp3_stream *stream, int *pfin,
                                     nghttp3_vec *vec, size_t veccnt) {
   nghttp3_ringbuf *outq = &stream->outq;
   size_t len = nghttp3_ringbuf_len(outq);
-  size_t i;
+  size_t i = stream->outq_idx;
   uint64_t offset = stream->outq_offset;
   size_t buflen;
   nghttp3_vec *vbegin = vec, *vend = vec + veccnt;
@@ -815,25 +818,27 @@ nghttp3_ssize nghttp3_stream_writev(nghttp3_stream *stream, int *pfin,
 
   assert(veccnt > 0);
 
-  for (i = stream->outq_idx; i < len; ++i) {
+  if (i < len) {
     tbuf = nghttp3_ringbuf_get(outq, i);
     buflen = nghttp3_buf_len(&tbuf->buf);
-    if (offset >= buflen) {
-      offset -= buflen;
-      continue;
+
+    if (offset < buflen) {
+      vec->base = tbuf->buf.pos + offset;
+      vec->len = (size_t)(buflen - offset);
+      ++vec;
+    } else {
+      /* This is the only case that satisfies offset >= buflen */
+      assert(0 == offset);
+      assert(0 == buflen);
     }
 
-    vec->base = tbuf->buf.pos + offset;
-    vec->len = (size_t)(buflen - offset);
-    ++vec;
     ++i;
-    break;
-  }
 
-  for (; i < len && vec != vend; ++i, ++vec) {
-    tbuf = nghttp3_ringbuf_get(outq, i);
-    vec->base = tbuf->buf.pos;
-    vec->len = nghttp3_buf_len(&tbuf->buf);
+    for (; i < len && vec != vend; ++i, ++vec) {
+      tbuf = nghttp3_ringbuf_get(outq, i);
+      vec->base = tbuf->buf.pos;
+      vec->len = nghttp3_buf_len(&tbuf->buf);
+    }
   }
 
   /* TODO Rework this if we have finished implementing HTTP
@@ -890,7 +895,7 @@ static int stream_pop_outq_entry(nghttp3_stream *stream,
     break;
   case NGHTTP3_BUF_TYPE_ALIEN:
     break;
-  default:
+  case NGHTTP3_BUF_TYPE_SHARED:
     assert(nghttp3_ringbuf_len(chunks));
 
     chunk = nghttp3_ringbuf_get(chunks, 0);
@@ -899,9 +904,17 @@ static int stream_pop_outq_entry(nghttp3_stream *stream,
     assert(chunk->end == tbuf->buf.end);
 
     if (chunk->last == tbuf->buf.last) {
-      nghttp3_buf_free(chunk, stream->mem);
+      if (nghttp3_buf_cap(chunk) == NGHTTP3_STREAM_MIN_CHUNK_SIZE) {
+        nghttp3_objalloc_chunk_release(stream->out_chunk_objalloc,
+                                       (nghttp3_chunk *)(void *)chunk->begin);
+      } else {
+        nghttp3_buf_free(chunk, stream->mem);
+      }
       nghttp3_ringbuf_pop_front(chunks);
     }
+    break;
+  default:
+    nghttp3_unreachable();
   };
 
   nghttp3_ringbuf_pop_front(&stream->outq);
@@ -925,7 +938,7 @@ int nghttp3_stream_add_ack_offset(nghttp3_stream *stream, uint64_t n) {
     if (tbuf->type == NGHTTP3_BUF_TYPE_ALIEN) {
       nack = nghttp3_min(offset, (uint64_t)buflen) - stream->ack_done;
       if (stream->callbacks.acked_data) {
-        rv = stream->callbacks.acked_data(stream, stream->node.nid.id, nack,
+        rv = stream->callbacks.acked_data(stream, stream->node.id, nack,
                                           stream->user_data);
         if (rv != 0) {
           return NGHTTP3_ERR_CALLBACK_FAILURE;
@@ -1206,8 +1219,7 @@ int nghttp3_stream_transit_rx_http_state(nghttp3_stream *stream,
   case NGHTTP3_HTTP_STATE_RESP_END:
     return NGHTTP3_ERR_MALFORMED_HTTP_MESSAGING;
   default:
-    assert(0);
-    abort();
+    nghttp3_unreachable();
   }
 }
 
