@@ -37,6 +37,7 @@
 #include "nghttp3_buf.h"
 #include "nghttp3_frame.h"
 #include "nghttp3_qpack.h"
+#include "nghttp3_objalloc.h"
 
 #define NGHTTP3_STREAM_MIN_CHUNK_SIZE 256
 
@@ -92,42 +93,46 @@ typedef struct nghttp3_stream_read_state {
 } nghttp3_stream_read_state;
 
 /* NGHTTP3_STREAM_FLAG_NONE indicates that no flag is set. */
-#define NGHTTP3_STREAM_FLAG_NONE 0x0000
+#define NGHTTP3_STREAM_FLAG_NONE 0x0000u
 /* NGHTTP3_STREAM_FLAG_TYPE_IDENTIFIED is set when a unidirectional
    stream type is identified. */
-#define NGHTTP3_STREAM_FLAG_TYPE_IDENTIFIED 0x0001
+#define NGHTTP3_STREAM_FLAG_TYPE_IDENTIFIED 0x0001u
 /* NGHTTP3_STREAM_FLAG_FC_BLOCKED indicates that stream is blocked by
    QUIC flow control. */
-#define NGHTTP3_STREAM_FLAG_FC_BLOCKED 0x0002
+#define NGHTTP3_STREAM_FLAG_FC_BLOCKED 0x0002u
 /* NGHTTP3_STREAM_FLAG_READ_DATA_BLOCKED indicates that application is
    temporarily unable to provide data. */
-#define NGHTTP3_STREAM_FLAG_READ_DATA_BLOCKED 0x0004
+#define NGHTTP3_STREAM_FLAG_READ_DATA_BLOCKED 0x0004u
 /* NGHTTP3_STREAM_FLAG_WRITE_END_STREAM indicates that application
    finished to feed outgoing data. */
-#define NGHTTP3_STREAM_FLAG_WRITE_END_STREAM 0x0008
+#define NGHTTP3_STREAM_FLAG_WRITE_END_STREAM 0x0008u
 /* NGHTTP3_STREAM_FLAG_QPACK_DECODE_BLOCKED indicates that stream is
    blocked due to QPACK decoding. */
-#define NGHTTP3_STREAM_FLAG_QPACK_DECODE_BLOCKED 0x0010
+#define NGHTTP3_STREAM_FLAG_QPACK_DECODE_BLOCKED 0x0010u
 /* NGHTTP3_STREAM_FLAG_READ_EOF indicates that remote endpoint sent
    fin. */
-#define NGHTTP3_STREAM_FLAG_READ_EOF 0x0020
+#define NGHTTP3_STREAM_FLAG_READ_EOF 0x0020u
 /* NGHTTP3_STREAM_FLAG_CLOSED indicates that QUIC stream was closed.
    nghttp3_stream object can still alive because it might be blocked
    by QPACK decoder. */
-#define NGHTTP3_STREAM_FLAG_CLOSED 0x0040
+#define NGHTTP3_STREAM_FLAG_CLOSED 0x0040u
 /* NGHTTP3_STREAM_FLAG_SHUT_WR indicates that any further write
    operation to a stream is prohibited. */
-#define NGHTTP3_STREAM_FLAG_SHUT_WR 0x0100
+#define NGHTTP3_STREAM_FLAG_SHUT_WR 0x0100u
 /* NGHTTP3_STREAM_FLAG_SHUT_RD indicates that a read-side stream is
    closed abruptly and any incoming and pending stream data is just
    discarded for a stream. */
-#define NGHTTP3_STREAM_FLAG_SHUT_RD 0x0200
+#define NGHTTP3_STREAM_FLAG_SHUT_RD 0x0200u
 /* NGHTTP3_STREAM_FLAG_SERVER_PRIORITY_SET indicates that server
    overrides stream priority. */
-#define NGHTTP3_STREAM_FLAG_SERVER_PRIORITY_SET 0x0400
+#define NGHTTP3_STREAM_FLAG_SERVER_PRIORITY_SET 0x0400u
 /* NGHTTP3_STREAM_FLAG_PRIORITY_UPDATE_RECVED indicates that server
    received PRIORITY_UPDATE frame for this stream. */
-#define NGHTTP3_STREAM_FLAG_PRIORITY_UPDATE_RECVED 0x0800
+#define NGHTTP3_STREAM_FLAG_PRIORITY_UPDATE_RECVED 0x0800u
+/* NGHTTP3_STREAM_FLAG_HTTP_ERROR indicates that
+   NGHTTP3_ERR_MALFORMED_HTTP_HEADER error is encountered while
+   processing incoming HTTP fields. */
+#define NGHTTP3_STREAM_FLAG_HTTP_ERROR 0x1000u
 
 typedef enum nghttp3_stream_http_state {
   NGHTTP3_HTTP_STATE_NONE,
@@ -190,60 +195,70 @@ typedef struct nghttp3_http_state {
   /* recv_content_length is the number of body bytes received so
      far. */
   int64_t recv_content_length;
-  uint16_t flags;
+  uint32_t flags;
   /* pri is a stream priority produced by nghttp3_pri_to_uint8. */
   uint8_t pri;
 } nghttp3_http_state;
 
 struct nghttp3_stream {
-  const nghttp3_mem *mem;
-  nghttp3_tnode node;
-  nghttp3_pq_entry qpack_blocked_pe;
-  nghttp3_stream_callbacks callbacks;
-  nghttp3_ringbuf frq;
-  nghttp3_ringbuf chunks;
-  nghttp3_ringbuf outq;
-  /* inq stores the stream raw data which cannot be read because
-     stream is blocked by QPACK decoder. */
-  nghttp3_ringbuf inq;
-  nghttp3_qpack_stream_context qpack_sctx;
-  /* conn is a reference to underlying connection.  It could be NULL
-     if stream is not a request stream. */
-  nghttp3_conn *conn;
-  void *user_data;
-  /* unsent_bytes is the number of bytes in outq not written yet */
-  uint64_t unsent_bytes;
-  /* outq_idx is an index into outq where next write is made. */
-  size_t outq_idx;
-  /* outq_offset is write offset relative to the element at outq_idx
-     in outq. */
-  uint64_t outq_offset;
-  /* ack_offset is offset acknowledged by peer relative to the first
-     element in outq. */
-  uint64_t ack_offset;
-  /* ack_done is the number of bytes notified to an application that
-     they are acknowledged inside the first outq element if it is of
-     type NGHTTP3_BUF_TYPE_ALIEN. */
-  uint64_t ack_done;
-  uint64_t unscheduled_nwrite;
-  nghttp3_stream_type type;
-  nghttp3_stream_read_state rstate;
-  /* error_code indicates the reason of closure of this stream. */
-  uint64_t error_code;
+  union {
+    struct {
+      const nghttp3_mem *mem;
+      nghttp3_objalloc *out_chunk_objalloc;
+      nghttp3_objalloc *stream_objalloc;
+      nghttp3_tnode node;
+      nghttp3_pq_entry qpack_blocked_pe;
+      nghttp3_stream_callbacks callbacks;
+      nghttp3_ringbuf frq;
+      nghttp3_ringbuf chunks;
+      nghttp3_ringbuf outq;
+      /* inq stores the stream raw data which cannot be read because
+         stream is blocked by QPACK decoder. */
+      nghttp3_ringbuf inq;
+      nghttp3_qpack_stream_context qpack_sctx;
+      /* conn is a reference to underlying connection.  It could be NULL
+         if stream is not a request stream. */
+      nghttp3_conn *conn;
+      void *user_data;
+      /* unsent_bytes is the number of bytes in outq not written yet */
+      uint64_t unsent_bytes;
+      /* outq_idx is an index into outq where next write is made. */
+      size_t outq_idx;
+      /* outq_offset is write offset relative to the element at outq_idx
+         in outq. */
+      uint64_t outq_offset;
+      /* ack_offset is offset acknowledged by peer relative to the first
+         element in outq. */
+      uint64_t ack_offset;
+      /* ack_done is the number of bytes notified to an application that
+         they are acknowledged inside the first outq element if it is of
+         type NGHTTP3_BUF_TYPE_ALIEN. */
+      uint64_t ack_done;
+      uint64_t unscheduled_nwrite;
+      nghttp3_stream_type type;
+      nghttp3_stream_read_state rstate;
+      /* error_code indicates the reason of closure of this stream. */
+      uint64_t error_code;
 
-  struct {
-    uint64_t offset;
-    nghttp3_stream_http_state hstate;
-  } tx;
+      struct {
+        uint64_t offset;
+        nghttp3_stream_http_state hstate;
+      } tx;
 
-  struct {
-    nghttp3_stream_http_state hstate;
-    nghttp3_http_state http;
-  } rx;
+      struct {
+        nghttp3_stream_http_state hstate;
+        nghttp3_http_state http;
+      } rx;
 
-  uint16_t flags;
+      uint16_t flags;
+    };
+
+    nghttp3_opl_entry oplent;
+  };
 };
 
+nghttp3_objalloc_def(stream, nghttp3_stream, oplent);
+
 typedef struct nghttp3_frame_entry {
   nghttp3_frame fr;
   union {
@@ -257,7 +272,9 @@ typedef struct nghttp3_frame_entry {
 } nghttp3_frame_entry;
 
 int nghttp3_stream_new(nghttp3_stream **pstream, int64_t stream_id,
-                       uint64_t seq, const nghttp3_stream_callbacks *callbacks,
+                       const nghttp3_stream_callbacks *callbacks,
+                       nghttp3_objalloc *out_chunk_objalloc,
+                       nghttp3_objalloc *stream_objalloc,
                        const nghttp3_mem *mem);
 
 void nghttp3_stream_del(nghttp3_stream *stream);
